/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.PathDiscovery;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import org.xml.sax.InputSource;

import java.io.ByteArrayInputStream;
import java.io.File;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.jsp.PageContext;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;


/**
 * <p>This implementation of the PathDiscovery and SiteDiscovery will attempt to create a static
 *  site map from either a filename or string source (both of which is an XML document).
 *  This class will parse the document to create a static map of Regex matchers (where each node
 *  can provide a regex matching).  If used as a PathDiscovery, this provides a static view of the
 *  site (ie no dynamic capabilities).</p>
 * <p>Thread Safety: Implementations of this interface should be thread safe.</p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
class XmlPathDiscovery implements SiteDiscovery, PathDiscovery {
    /**
     * <p>This static constant represents the session scope attribute name that will contain the
     *  site map. This constant will be used by this class to retrieve the site map. The site map
     *  will be a Map with NodeMatcher as the key and a List of NodeMatcher as the value and can
     *  be empty. This map represents the NodeMatcher parent to it's children NodeMatcher(s) if any.
     *  The list can be empty if there are no children of the parent (ie a leaf node)</p>
     */
    public static final String CURRENT_SITEMAP = XmlPathDiscovery.class.getName() + ".siteMap";

    /**
     * <p>Represents the source of the site data.  The source will be set by the constructor, is
     *  immutable and will never be null but may be an empty (trim'd) string.  This variable is
     *  referencable by the getSiteMap method.</p>
     */
    private final String source;

    /**
     * <p>Represents whether the source variable represents a file (true) or a raw xml string
     *  (false).  The isFile will be set by the constructor and is immutable.  This variable is
     *  referencable by the getSiteMap method.</p>
     */
    private final boolean isFile;

    /**
     * <p>Constructs the XmlPathDiscovery from the given filename.</p>
     *
     *
     * @param filename A non-null, non-empty (trim'd) string representing the file to use
     * @throws IllegalArgumentException if the filename is a null or empty (trim'd) string
     */
    public XmlPathDiscovery(String filename) {
        this(filename, true);
    }

    /**
     * <p>Constructs the XmlPathDiscovery from the given source and whether that source is a
     * direct XML string or a filename.</p>
     *
     *
     * @param source A non-null, non-empty (trim'd) string representing the source
     * @param isFile Whether the source is a file (true) or a string (false)
     * @throws IllegalArgumentException if the source is null or an emtpy (trim'd) string
     */
    public XmlPathDiscovery(String source, boolean isFile) {
        if (source == null) {
            throw new IllegalArgumentException("source cannot be null.");
        }

        if (source.trim().length() == 0) {
            throw new IllegalArgumentException("source cannot be empty string.");
        }

        this.source = source;
        this.isFile = isFile;
    }

    /**
     * <p>Returns a map of NodeMatcher keys with a List of NodeMatcher elements as the value.
     *   The key NodeMatcher represents the parent and the value (the list) represents the
     *   child values.  The value can be null or an empty list to represent no child elements
     *   (ie it'a s leaf node).</p>
     *
     *
     * @return A non-null map of parent NodeMatcher to a list of NodeMatcher children
     * @throws BreadCrumbException if parsing of data source failed
     */
    public Map getSiteMap() throws BreadCrumbException {
        try {
            HashMap map = new HashMap();
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document document = null;

            if (isFile) {
                document = builder.parse(new File(source));
            } else {
                byte[] bytes = new byte[source.length()];

                for (int i = 0; i < source.length(); ++i) {
                    bytes[i] = (byte) (source.charAt(i));
                }

                document = builder.parse(new InputSource(new ByteArrayInputStream(bytes)));
            }

            Node node = document.getFirstChild();
            map.put(node, node);

            return map;
        } catch (Exception e) {
            throw new BreadCrumbException("Failed to get map.", e);
        }
    }

    /**
     * <p>This method attempts to determine the page from the current node to the root and return
     *  that path based on the current site map.</p>
     *
     *
     * @param currentNode A non-null current node
     * @param pageContext A non-null page context
     * @return A null if the path can't be found or a non-empty array representing the path
     *  from root node (0th index) to current node (size-1th index)
     * @throws IllegalArgumentException if any arg is null
     * @throws BreadCrumbException if an exception occurred while processing
     */
    public BreadCrumbNode[] getPath(BreadCrumbNode currentNode, PageContext pageContext)
        throws BreadCrumbException {
        if (currentNode == null) {
            throw new IllegalArgumentException("currentNode cannot be null.");
        }

        if (pageContext == null) {
            throw new IllegalArgumentException("pageContext cannot be null.");
        }

        return null;
    }
}
